package xyz.zhouxy.plusone.exception.handler;

import org.springframework.context.support.DefaultMessageSourceResolvable;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.dao.DataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import lombok.extern.slf4j.Slf4j;
import xyz.zhouxy.plusone.util.RestfulResult;

/**
 * 默认异常的处理器
 *
 * <p>
 * 对 {@link IllegalArgumentException} 异常做了写默认处理。
 * </p>
 *
 * <p>
 * 将异常信息返回给前端时，隐藏掉数据库异常信息中关于数据库结构、SQL 语句的描述。
 * </p>
 *
 * <p>
 * 为了避免因为 {@link AllExceptionHandler} 而失效，
 * {@code Order} 设置为了 {@value Ordered.LOWEST_PRECEDENCE - 1}。
 * 一般 Spring Bean 的 Order 应该是 Ordered.LOWEST_PRECEDENCE，
 * 所以如果想让自定义的异常处理器与本处理器有冲突，希望覆盖本处理器的行为，
 * 需要添加 {@link Order} 注解，将 order 修改为比 {@code Ordered.LOWEST_PRECEDENCE - 1}
 * 更小的值，
 * 如 0 或者 1，甚至是负数。
 * </p>
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@RestControllerAdvice
@Order(Ordered.LOWEST_PRECEDENCE - 1)
@Slf4j
public class DefaultExceptionHandler extends BaseExceptionHandler {

    public DefaultExceptionHandler(ExceptionInfoHolder exceptionInfoHolder) {
        super(exceptionInfoHolder);
        set(IllegalArgumentException.class, 4010000, "格式错误", HttpStatus.FORBIDDEN);
        set(DataAccessException.class, 6030000, "数据库错误", HttpStatus.INTERNAL_SERVER_ERROR, true);
        set(MethodArgumentNotValidException.class,
                4040401,
                e -> ((MethodArgumentNotValidException) e).getAllErrors()
                        .stream()
                        .map(DefaultMessageSourceResolvable::getDefaultMessage)
                        .toList()
                        .toString(),
                HttpStatus.BAD_REQUEST);
    }

    @ExceptionHandler({
            IllegalArgumentException.class,
            DataAccessException.class,
            MethodArgumentNotValidException.class
    })
    public ResponseEntity<RestfulResult> handleException(Exception e) {
        log.error(e.getMessage(), e);
        return buildExceptionResponse(e);
    }
}
