package xyz.zhouxy.plusone.sms;

import com.tencentcloudapi.common.Credential;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import com.tencentcloudapi.common.profile.ClientProfile;
import com.tencentcloudapi.common.profile.HttpProfile;
import com.tencentcloudapi.sms.v20210111.SmsClient;
import com.tencentcloudapi.sms.v20210111.models.SendSmsRequest;
import org.springframework.stereotype.Service;

import lombok.extern.slf4j.Slf4j;

/**
 * 使用腾讯 SMS 服务
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Service
@Slf4j
public class TencentSmsServiceImpl implements SmsService {

    private final SmsProperties properties;
    private final Credential credential;

    public TencentSmsServiceImpl(SmsProperties properties) {
        this.properties = properties;
        SmsCredentialProperties smsCredential = properties.getCredential();
        this.credential = new Credential(smsCredential.getSecretId(), smsCredential.getSecretKey());
    }

    @Override
    public void sendCodeMessage(String code, String phoneNumber) {
        sendCodeMessage("ZhouXY", code, phoneNumber);
    }

    @Override
    public void sendCodeMessage(String signName, String code, String phoneNumber) {
        String[] phoneNums = {"+86" + phoneNumber};
        sendMessage(signName, "code", phoneNums, code, "10");
    }

    public void sendMessage(String signName, String action, String[] phoneNumberSet, String... templateParamSet) {
        try {

            SmsClient client = getClient();
            SendSmsRequest req = getSendSmsRequest(signName,
                properties.getTemplates().get(action),
                phoneNumberSet,
                templateParamSet);

            /*
             * 通过 client 对象调用 SendSms 方法发起请求。注意请求方法名与请求对象是对应的 返回的 res 是一个
             * SendSmsResponse 类的实例，与请求对象对应
             */
            // var res = client.SendSms(req);
            client.SendSms(req);

            // 输出json格式的字符串回包
            // System.out.println(SendSmsResponse.toJsonString(res));

            // 也可以取出单个值，你可以通过官网接口文档或跳转到response对象的定义处查看返回字段的定义
            // System.out.println(res.getRequestId());

        } catch (TencentCloudSDKException e) {
            log.error(e.getMessage(), e);
        }
    }

    private SendSmsRequest getSendSmsRequest(String signName,
                                             String templateId,
                                             String[] phoneNumberSet,
                                             String... templateParamSet) {
        SendSmsRequest req = new SendSmsRequest();

        String sdkAppId = properties.getAppId();
        req.setSmsSdkAppId(sdkAppId);
        req.setSignName(signName);

        /* 国际/港澳台短信 SenderId: 国内短信填空，默认未开通，如需开通请联系 [sms helper] */
        String senderId = "";
        req.setSenderId(senderId);

        /* 用户的 session 内容: 可以携带用户侧 ID 等上下文信息，server 会原样返回 */
        // String sessionContext = "xxx";
        // req.setSessionContext(sessionContext);

        /* 短信号码扩展号: 默认未开通，如需开通请联系 [sms helper] */
        String extendCode = "";
        req.setExtendCode(extendCode);

        /* 模板 ID: 必须填写已审核通过的模板 ID。模板ID可登录 [短信控制台] 查看 */
        req.setTemplateId(templateId);

        /*
         * 下发手机号码，采用 E.164 标准，+[国家或地区码][手机号] 示例如：+8613711112222， 其中前面有一个+号
         * ，86为国家码，13711112222为手机号，最多不要超过200个手机号
         */
        req.setPhoneNumberSet(phoneNumberSet);

        /* 模板参数: 若无模板参数，则设置为空 */
        req.setTemplateParamSet(templateParamSet);
        return req;
    }

    private SmsClient getClient() {
        String region = properties.getRegion();
        ClientProfile clientProfile = getClientProfile();
        return new SmsClient(credential, region, clientProfile);
    }

    private ClientProfile getClientProfile() {
        HttpProfile httpProfile = getHttpProfile();
        /*
         * 非必要步骤: 实例化一个客户端配置对象，可以指定超时时间等配置
         */
        ClientProfile clientProfile = new ClientProfile();
        String signMethod = properties.getClient().getSignMethod();
        if (signMethod != null) {
            clientProfile.setSignMethod(signMethod);
        } else {
            clientProfile.setSignMethod("HmacSHA256");
        }
        clientProfile.setHttpProfile(httpProfile);
        return clientProfile;
    }

    private HttpProfile getHttpProfile() {
        HttpProfile httpProfile = new HttpProfile();
        SmsHttpProperties smsHttp = properties.getClient().getHttp();
        if (smsHttp != null) {
            if (smsHttp.getReqMethod() != null) {
                httpProfile.setReqMethod(smsHttp.getReqMethod());
            }
            if (smsHttp.getConnTimeout() != null) {
                httpProfile.setConnTimeout(60);
            }
            SmsProxyProperties proxy = smsHttp.getProxy();
            if (proxy != null && proxy.getHost() != null && proxy.getPort() != null) {// 设置代理
                httpProfile.setProxyHost(proxy.getHost());
                httpProfile.setProxyPort(proxy.getPort());
            }
        }
        return httpProfile;
    }
}
