package xyz.zhouxy.plusone.system.application.service;

import static xyz.zhouxy.plusone.system.constant.AuthLogic.adminAuthLogic;

import java.util.List;
import java.util.Objects;

import javax.validation.Valid;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PathVariable;

import xyz.zhouxy.plusone.system.application.exception.AccountRegisterException;
import xyz.zhouxy.plusone.system.application.query.AccountQueries;
import xyz.zhouxy.plusone.system.application.query.params.AccountQueryParams;
import xyz.zhouxy.plusone.system.application.query.result.AccountDetails;
import xyz.zhouxy.plusone.system.application.query.result.AccountOverview;
import xyz.zhouxy.plusone.system.application.service.command.CreateAccountCommand;
import xyz.zhouxy.plusone.system.application.service.command.UpdateAccountCommand;
import xyz.zhouxy.plusone.system.domain.model.account.Account;
import xyz.zhouxy.plusone.system.domain.model.account.AccountInfo;
import xyz.zhouxy.plusone.system.domain.model.account.AccountRepository;
import xyz.zhouxy.plusone.system.domain.model.account.Email;
import xyz.zhouxy.plusone.system.domain.model.account.MobilePhone;
import xyz.zhouxy.plusone.system.domain.model.account.Username;
import xyz.zhouxy.plusone.util.AssertResult;
import xyz.zhouxy.plusone.util.PageDTO;

/**
 * 账号管理
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Service
@Transactional
public class AccountManagementService {

    private final AccountRepository accountRepository;
    private final AccountQueries accountQueries;

    public AccountManagementService(AccountRepository accountRepository, AccountQueries accountQueries) {
        this.accountRepository = accountRepository;
        this.accountQueries = accountQueries;
    }

    public void createAccount(@Valid CreateAccountCommand command) {
        String username = command.getUsername();
        if (accountRepository.existsUsername(Username.of(username))) {
            throw AccountRegisterException.usernameAlreadyExists(username);
        }
        String email = command.getEmail();
        if (StringUtils.hasText(email) && accountRepository.existsEmail(Email.of(email))) {
            throw AccountRegisterException.emailAlreadyExists(email);
        }
        String mobilePhone = command.getMobilePhone();
        if (StringUtils.hasText(mobilePhone) && accountRepository.existsMobilePhone(MobilePhone.of(mobilePhone))) {
            throw AccountRegisterException.mobilePhoneAlreadyExists(mobilePhone);
        }
        Account account = Account.newInstance(
                username,
                email,
                mobilePhone,
                command.getPassword(),
                command.getPasswordConfirmation(),
                command.getStatus(),
                command.getRoleRefs(),
                AccountInfo.of(command.getNickname(), command.getAvatar(), command.getSex()),
                adminAuthLogic.getLoginIdAsLong());
        accountRepository.save(account);
    }

    public void deleteAccounts(List<Long> ids) {
        Account accountToDelete;
        for (var id : ids) {
            accountToDelete = accountRepository.find(id);
            AssertResult.nonNull(accountToDelete);
            accountRepository.delete(accountToDelete);
        }
    }

    public void updateAccountInfo(Long id, @Valid UpdateAccountCommand command) {
        Assert.isTrue(Objects.equals(id, command.getId()), "参数错误: id 不匹配");
        Account account = accountRepository.find(id);
        AssertResult.nonNull(account, "该账号不存在");
        account.setAccountInfo(command.getNickname(), command.getAvatar(), command.getSex());
        account.setUpdatedBy(adminAuthLogic.getLoginIdAsLong());
        accountRepository.save(account);
    }

    @Transactional(propagation = Propagation.SUPPORTS)
    public PageDTO<AccountOverview> queryAccountOverviewList(AccountQueryParams queryParams) {
        return accountQueries.queryAccountOverviewPage(queryParams);
    }

    @Transactional(propagation = Propagation.SUPPORTS)
    public AccountDetails queryAccountDetails(@PathVariable("accountId") Long accountId) {
        var accountDetails = accountQueries.queryAccountDetails(accountId);
        AssertResult.nonNull(accountDetails);
        return accountDetails;
    }
}
