package xyz.zhouxy.plusone.system.application.service;

import static xyz.zhouxy.plusone.system.constant.AuthLogic.adminAuthLogic;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import xyz.zhouxy.plusone.exception.InvalidInputException;
import xyz.zhouxy.plusone.system.application.common.util.PrincipalType;
import xyz.zhouxy.plusone.system.application.common.util.PrincipalUtil;
import xyz.zhouxy.plusone.system.application.exception.AccountLoginException;
import xyz.zhouxy.plusone.system.application.query.AccountQueries;
import xyz.zhouxy.plusone.system.application.query.result.LoginInfoViewObject;
import xyz.zhouxy.plusone.system.application.service.command.LoginByOtpCommand;
import xyz.zhouxy.plusone.system.application.service.command.LoginByPasswordCommand;
import xyz.zhouxy.plusone.system.domain.model.account.Account;
import xyz.zhouxy.plusone.system.domain.model.account.AccountRepository;
import xyz.zhouxy.plusone.system.domain.model.account.Email;
import xyz.zhouxy.plusone.system.domain.model.account.MobilePhone;
import xyz.zhouxy.plusone.system.domain.model.account.Principal;
import xyz.zhouxy.plusone.system.domain.model.account.Username;
import xyz.zhouxy.plusone.validator.ValidateDto;

/**
 * Admin 账号登录
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Service
@Transactional
public class AdminLoginService {

    private final AccountRepository accountRepository;
    private final AccountQueries accountQueries;
    private final MailAndSmsVerifyService mailAndSmsVerifyService;

    AdminLoginService(AccountRepository accountRepository, AccountQueries accountQueries,
            MailAndSmsVerifyService mailAndSmsVerifyService) {
        this.accountRepository = accountRepository;
        this.accountQueries = accountQueries;
        this.mailAndSmsVerifyService = mailAndSmsVerifyService;
    }

    @ValidateDto
    public LoginInfoViewObject loginByPassword(LoginByPasswordCommand command) {
        Principal principal = PrincipalUtil.getPrincipal(command.getPrincipal());
        Account account;
        if (principal instanceof Email) {
            account = accountRepository.findByEmail((Email) principal);
        } else if (principal instanceof MobilePhone) {
            account = accountRepository.findByMobilePhone((MobilePhone) principal);
        } else {
            account = accountRepository.findByUsername((Username) principal);
        }

        if (account == null) {
            throw AccountLoginException.accountNotExistException();
        }
        @SuppressWarnings("null")
        boolean isPasswordCorrect = account.checkPassword(command.getPassword());
        if (!isPasswordCorrect) {
            throw AccountLoginException.passwordErrorException();
        }
        adminAuthLogic.login(account.getId().orElseThrow(), command.isRememberMe());

        var accountDetails = accountQueries.queryAccountDetails(account.getId().orElseThrow());
        return LoginInfoViewObject.of(adminAuthLogic.getTokenValue(), accountDetails);
    }

    @ValidateDto
    public LoginInfoViewObject loginByOtp(LoginByOtpCommand command) {
        String principal = command.getPrincipal();
        PrincipalType principalType = PrincipalUtil.getPrincipalType(principal);
        String otp = command.getOtp();
        boolean rememberMe = command.isRememberMe();

        Account account;
        if (principalType == PrincipalType.EMAIL) {
            account = accountRepository.findByEmail(Email.of(principal));
        } else if (principalType == PrincipalType.MOBILE_PHONE) {
            account = accountRepository.findByMobilePhone(MobilePhone.of(principal));
        } else {
            throw InvalidInputException.unsupportedPrincipalTypeException("输入邮箱地址或手机号");
        }

        if (account == null) {
            throw AccountLoginException.accountNotExistException();
        }
        mailAndSmsVerifyService.checkOtp(principal, otp);
        adminAuthLogic.login(account.getId().orElseThrow(), rememberMe);

        var accountDetails = accountQueries.queryAccountDetails(account.getId().orElseThrow());
        return LoginInfoViewObject.of(adminAuthLogic.getTokenValue(), accountDetails);
    }

    public void sendOtp(String principal) {
        Principal emailOrMobilePhone = PrincipalUtil.getEmailOrMobilePhone(principal);
        if (emailOrMobilePhone instanceof Email) {
            mailAndSmsVerifyService.sendOtpToEmail((Email) emailOrMobilePhone);
        } else {
            mailAndSmsVerifyService.sendOtpToMobilePhone((MobilePhone) emailOrMobilePhone);
        }
    }
}
