package xyz.zhouxy.plusone.system.application.service;

import java.util.Objects;

import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import cn.hutool.core.util.RandomUtil;
import xyz.zhouxy.plusone.mail.MailService;
import xyz.zhouxy.plusone.sms.SmsService;
import xyz.zhouxy.plusone.system.application.exception.AccountLoginException;
import xyz.zhouxy.plusone.system.application.exception.AccountRegisterException;
import xyz.zhouxy.plusone.system.domain.model.account.AccountRepository;
import xyz.zhouxy.plusone.system.domain.model.account.Email;
import xyz.zhouxy.plusone.system.domain.model.account.MobilePhone;

/**
 * 邮箱和短信的验证服务
 */
@Service
public class MailAndSmsVerifyService {
    private static final int CODE_LENGTH = 6;
    private final AccountRepository accountRepository;
    private final MailService mailService;
    private final SmsService smsService;
    private final StringRedisTemplate redisTemplate;

    public MailAndSmsVerifyService(AccountRepository accountRepository, MailService mailService, SmsService smsService,
            StringRedisTemplate redisTemplate) {
        this.accountRepository = accountRepository;
        this.mailService = mailService;
        this.smsService = smsService;
        this.redisTemplate = redisTemplate;
    }

    /**
     * 发送一次性密码到邮箱
     *
     * @param email 要求邮箱必须已注册
     */
    public void sendOtpToEmail(Email email) {
        Assert.isTrue(accountRepository.existsEmail(email), "该邮箱未绑定任何帐号");
        var otp = generateCode();
        mailService.sendCodeMail(otp, email.value());
        redisTemplate.opsForValue().set("OTP-" + email.value(), otp);
    }

    /**
     * 发送一次性密码到手机号
     *
     * @param mobilePhone 要求手机号必须已注册
     */
    public void sendOtpToMobilePhone(MobilePhone mobilePhone) {
        Assert.isTrue(accountRepository.existsMobilePhone(mobilePhone), "该手机号未绑定任何帐号");
        var otp = generateCode();
        smsService.sendCodeMessage(otp, mobilePhone.value());
        redisTemplate.opsForValue().set("OTP-" + mobilePhone.value(), otp);
    }

    /**
     * 发送校验码到邮箱
     *
     * @param email 要求邮箱必须未注册
     */
    public void sendCodeToEmail(Email email) {
        Assert.isTrue(!accountRepository.existsEmail(email), "该邮箱未绑定任何帐号");
        var code = generateCode();
        mailService.sendCodeMail(code, email.value());
        redisTemplate.opsForValue().set("Code-" + email.value(), code);
    }

    /**
     * 发送校验码到手机号
     *
     * @param mobilePhone 要求手机号必须未注册
     */
    public void sendCodeToMobilePhone(MobilePhone mobilePhone) {
        Assert.isTrue(!accountRepository.existsMobilePhone(mobilePhone), "该手机号未绑定任何帐号");
        var code = generateCode();
        smsService.sendCodeMessage(code, mobilePhone.value());
        redisTemplate.opsForValue().set("Code-" + mobilePhone.value(), code);
    }

    public void checkOtp(String emailOrMobilePhone, String otp) {
        String key = "OTP-" + emailOrMobilePhone;
        String otpInRedis = redisTemplate.opsForValue().get(key);
        if (otpInRedis == null) {
            throw AccountLoginException.otpNotExistsException();
        }
        if (!Objects.equals(otpInRedis, otp)) {
            throw AccountLoginException.otpErrorException();
        }
        redisTemplate.delete(key);
    }

    public void checkCode(String emailOrMobilePhone, String code) {
        String key = "Code-" + emailOrMobilePhone;
        String codeInRedis = redisTemplate.opsForValue().get(key);
        if (codeInRedis == null) {
            throw AccountRegisterException.codeNotExistsException();
        }
        if (!Objects.equals(codeInRedis, code)) {
            throw AccountRegisterException.codeErrorException();
        }
        redisTemplate.delete(key);
    }

    private static String generateCode() {
        return RandomUtil.randomString(CODE_LENGTH);
    }

}
