package xyz.zhouxy.plusone.system.application.service;

import java.util.Set;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import xyz.zhouxy.plusone.exception.InvalidInputException;
import xyz.zhouxy.plusone.system.application.common.util.PrincipalType;
import xyz.zhouxy.plusone.system.application.common.util.PrincipalUtil;
import xyz.zhouxy.plusone.system.application.exception.AccountRegisterException;
import xyz.zhouxy.plusone.system.application.service.command.RegisterAccountCommand;
import xyz.zhouxy.plusone.system.domain.model.account.Account;
import xyz.zhouxy.plusone.system.domain.model.account.AccountInfo;
import xyz.zhouxy.plusone.system.domain.model.account.AccountRepository;
import xyz.zhouxy.plusone.system.domain.model.account.AccountStatus;
import xyz.zhouxy.plusone.system.domain.model.account.Email;
import xyz.zhouxy.plusone.system.domain.model.account.MobilePhone;
import xyz.zhouxy.plusone.system.domain.model.account.Password;
import xyz.zhouxy.plusone.system.domain.model.account.Username;

/**
 * 注册账号服务
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Service
@Transactional
public class RegisterAccountService {

    private static final long DEFAULT_ROLE_ID = 1L;

    private final AccountRepository accountRepository;
    private final MailAndSmsVerifyService verifyService;

    public RegisterAccountService(AccountRepository accountRepository, MailAndSmsVerifyService verifyService) {
        this.accountRepository = accountRepository;
        this.verifyService = verifyService;
    }

    public void registerAccount(RegisterAccountCommand command) {
        String username = command.getUsername();
        var existsUsername = accountRepository.existsUsername(Username.of(username));
        if (existsUsername) {
            throw AccountRegisterException.usernameAlreadyExists(username);
        }
        // 1. 确定是使用邮箱地址还是手机号进行注册
        String emailOrMobilePhone = command.getEmailOrMobilePhone();
        if (emailOrMobilePhone == null) {
            throw new IllegalArgumentException("邮箱地址或手机号不能为空");
        }
        PrincipalType principalType = PrincipalUtil.getPrincipalType(emailOrMobilePhone);

        // 2. 确定该邮箱地址或手机号是否存在，比对校验码
        Email email = null;
        MobilePhone mobilePhone = null;
        boolean isExists;

        if (principalType == PrincipalType.EMAIL) {
            email = Email.of(emailOrMobilePhone);
            isExists = accountRepository.existsEmail(email);
            if (isExists) {
                throw AccountRegisterException.emailAlreadyExists(email.value());
            }
        } else if (principalType == PrincipalType.MOBILE_PHONE) {
            mobilePhone = MobilePhone.of(emailOrMobilePhone);
            isExists = accountRepository.existsMobilePhone(mobilePhone);
            if (isExists) {
                throw AccountRegisterException.emailAlreadyExists(mobilePhone.value());
            }
        } else {
            throw InvalidInputException.unsupportedPrincipalTypeException();
        }

        verifyService.checkCode(emailOrMobilePhone, command.getCode());

        Account accountToSave = Account.register(
                Username.of(username),
                email,
                mobilePhone,
                Password.newPassword(command.getPassword(), command.getPasswordConfirmation()),
                AccountStatus.AVAILABLE,
                Set.of(DEFAULT_ROLE_ID),
                AccountInfo.of(command.getNickname(), command.getAvatar(), command.getSex()));
        accountRepository.save(accountToSave);
    }

    public void sendCode(String principal) {
        PrincipalType principalType = PrincipalUtil.getPrincipalType(principal);
        if (principalType == PrincipalType.EMAIL) {
            verifyService.sendCodeToEmail(Email.of(principal));
        } else if (principalType == PrincipalType.MOBILE_PHONE) {
            verifyService.sendCodeToMobilePhone(MobilePhone.of(principal));
        } else {
            throw InvalidInputException.unsupportedPrincipalTypeException();
        }
    }
}
