package xyz.zhouxy.plusone.system.application.service;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import javax.validation.Valid;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import xyz.zhouxy.plusone.system.application.query.RoleQueries;
import xyz.zhouxy.plusone.system.application.query.params.RoleQueryParams;
import xyz.zhouxy.plusone.system.application.query.result.RoleOverview;
import xyz.zhouxy.plusone.system.application.service.command.CreateRoleCommand;
import xyz.zhouxy.plusone.system.application.service.command.UpdateRoleCommand;
import xyz.zhouxy.plusone.system.domain.model.menu.MenuRepository;
import xyz.zhouxy.plusone.system.domain.model.role.ActionRef;
import xyz.zhouxy.plusone.system.domain.model.role.MenuRef;
import xyz.zhouxy.plusone.system.domain.model.role.Role;
import xyz.zhouxy.plusone.system.domain.model.role.RoleRepository;

/**
 * 角色管理服务
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Service
@Transactional
public class RoleManagementService {

    private final RoleRepository _roleRepository;
    private final MenuRepository _menuRepository;
    private final RoleQueries _roleQueries;

    public RoleManagementService(RoleRepository roleRepository, MenuRepository menuRepository,
            RoleQueries roleQueries) {
        _roleRepository = roleRepository;
        _menuRepository = menuRepository;
        _roleQueries = roleQueries;
    }

    public void createRole(@Valid CreateRoleCommand command) {
        Set<MenuRef> menuRefs = _menuRepository.findByIdIn(command.getMenus())
                .stream()
                .map(menu -> new MenuRef(menu.getId().orElseThrow()))
                .collect(Collectors.toSet());
        Set<ActionRef> permissionRefs = _menuRepository.findPermissionsByIdIn(command.getPermissions())
                .stream()
                .map(permission -> new ActionRef(permission.getId().orElseThrow()))
                .collect(Collectors.toSet());
        Role roleToCreate = Role.newInstance(
                command.getName(),
                command.getIdentifier(),
                command.getStatus(),
                command.getRemarks(),
                menuRefs,
                permissionRefs);
        _roleRepository.save(roleToCreate);
    }

    public void updateRole(@Valid UpdateRoleCommand command) {
        Long roleId = command.getId();
        Role roleToUpdate = _roleRepository.find(roleId);
        roleToUpdate.update(
                command.getName(),
                command.getIdentifier(),
                command.getStatus(),
                command.getRemarks(),
                Set.copyOf(_menuRepository.findByIdIn(command.getMenus())),
                Set.copyOf(_menuRepository.findPermissionsByIdIn(command.getPermissions())));
        _roleRepository.save(roleToUpdate);
    }

    public void delete(Long id) {
        Role role = _roleRepository.find(id);
        _roleRepository.delete(role);
    }

    @Transactional(propagation = Propagation.SUPPORTS)
    public boolean exists(Long id) {
        return _roleRepository.exists(id);
    }

    @Transactional(propagation = Propagation.SUPPORTS)
    public Role findById(Long id) {
        return _roleRepository.find(id);
    }

    @Transactional(propagation = Propagation.SUPPORTS)
    public List<RoleOverview> query(RoleQueryParams params) {
        return _roleQueries.query(params);
    }
}
