package xyz.zhouxy.plusone.system.domain.model.account;

import java.net.URL;
import java.util.HashSet;
import java.util.Optional;
import java.util.Set;

import javax.annotation.Nonnull;

import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import xyz.zhouxy.plusone.domain.AggregateRoot;
import xyz.zhouxy.plusone.domain.IWithVersion;
import xyz.zhouxy.plusone.exception.UserOperationException;
import xyz.zhouxy.plusone.system.domain.event.AccountCreated;
import xyz.zhouxy.plusone.system.domain.event.AccountLocked;
import xyz.zhouxy.plusone.system.domain.event.AccountPasswordChanged;
import xyz.zhouxy.plusone.system.domain.event.AccountRecovered;
import xyz.zhouxy.plusone.system.domain.event.AccountRolesBound;
import xyz.zhouxy.plusone.system.domain.event.EmailChanged;
import xyz.zhouxy.plusone.system.domain.event.MobilePhoneChanged;
import xyz.zhouxy.plusone.system.domain.event.UsernameChanged;

/**
 * 聚合根：账号
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@ToString
public class Account extends AggregateRoot<Long> implements IWithVersion {

    // ===================== 字段 ====================
    private Long id;
    private @Getter Username username;
    private @Getter Email email;
    private @Getter MobilePhone mobilePhone;
    private Password password;
    private @Getter AccountStatus status;
    private @Getter AccountInfo accountInfo;
    private Set<Long> roleRefs = new HashSet<>();

    private @Getter Long createdBy;
    private @Getter @Setter Long updatedBy;
    private @Getter long version;

    public void setUsername(Username username) {
        this.username = username;
        addDomainEvent(new UsernameChanged(this));
    }

    public void setEmail(Email email) {
        this.email = email;
        addDomainEvent(new EmailChanged(this));
    }

    public void setMobilePhone(MobilePhone mobilePhone) {
        this.mobilePhone = mobilePhone;
        addDomainEvent(new MobilePhoneChanged(this));
    }

    public void changePassword(String newPassword, String passwordConfirmation) {
        this.password = Password.newPassword(newPassword, passwordConfirmation);
        addDomainEvent(new AccountPasswordChanged(this));
    }

    /**
     * 锁定账号。如当前账号已锁定，则抛出 UserOperationException 异常
     *
     * @see UserOperationException
     */
    public void lockAccount() {
        if (this.status == AccountStatus.LOCKED) {
            throw UserOperationException
                    .invalidOperation(String.format("账号 %d 的状态为：%s，无法锁定", this.id, this.status.getName()));
        }
        this.status = AccountStatus.LOCKED;
        addDomainEvent(new AccountLocked(this));
    }

    /**
     * 恢复账号状态。如当前用户可用，则抛出 UserOperationException 异常
     *
     * @see UserOperationException
     */
    public void recoveryAccount() {
        if (this.status == AccountStatus.AVAILABLE) {
            throw UserOperationException
                    .invalidOperation(String.format("账号 %d 的状态为：%s，无法恢复", this.id, this.status.getName()));
        }
        this.status = AccountStatus.AVAILABLE;
        addDomainEvent(new AccountRecovered(this));
    }

    public void setAccountInfo(AccountInfo accountInfo) {
        this.accountInfo = accountInfo;
    }

    public void setAccountInfo(Nickname nickname, URL avatar, Sex sex) {
        this.accountInfo = AccountInfo.of(nickname, avatar, sex);
    }

    public void setAccountInfo(String nickname, String avatar, Sex sex) {
        this.accountInfo = AccountInfo.of(nickname, avatar, sex);
    }

    /**
     * 绑定角色
     *
     * @param roleRefs 角色 id 集合
     */
    public void bindRoles(Set<Long> roleRefs) {
        this.roleRefs.clear();
        this.roleRefs.addAll(roleRefs);
        addDomainEvent(new AccountRolesBound(this));
    }

    public boolean checkPassword(@Nonnull String password) {
        return this.password.check(password);
    }

    // ===================== 实例化 ====================

    Account(Long id,
            Username username,
            Email email,
            MobilePhone mobilePhone,
            Password password,
            AccountStatus status,
            AccountInfo accountInfo,
            Set<Long> roleRefs,
            Long createdBy,
            Long updatedBy,
            long version) {
        this.id = id;
        this.username = username;
        this.email = email;
        this.mobilePhone = mobilePhone;
        this.password = password;
        this.status = status;
        this.accountInfo = accountInfo;
        this.bindRoles(roleRefs);
        this.createdBy = createdBy;
        this.updatedBy = updatedBy;
        this.version = version;
    }

    public static Account newInstance(
            Username username,
            Email email,
            MobilePhone mobilePhone,
            Password password,
            AccountStatus status,
            Set<Long> roleRefs,
            AccountInfo accountInfo,
            Long createdBy) {
        var newInstance = new Account(null, username, email, mobilePhone, password, status, accountInfo, roleRefs,
                createdBy, null, 0);
        newInstance.addDomainEvent(new AccountCreated(newInstance));
        return newInstance;
    }

    public static Account register(
            Username username,
            Email email,
            MobilePhone mobilePhone,
            Password password,
            AccountStatus status,
            Set<Long> roleRefs,
            AccountInfo accountInfo) {
        var newInstance = new Account(null, username, email, mobilePhone, password, status, accountInfo, roleRefs,
                0L, null, 0);
        newInstance.addDomainEvent(new AccountCreated(newInstance));
        return newInstance;
    }

    Account(Long id,
            String username,
            String email,
            String mobilePhone,
            Password password,
            AccountStatus status,
            AccountInfo accountInfo,
            Set<Long> roleRefs,
            Long createdBy,
            Long updatedBy,
            long version) {
        this(id, Username.of(username), Email.ofNullable(email), MobilePhone.ofNullable(mobilePhone),
                password, status, accountInfo, roleRefs, createdBy, updatedBy, version);
    }

    public static Account newInstance(
            String username,
            String email,
            String mobilePhone,
            String password,
            String passwordConfirmation,
            AccountStatus status,
            Set<Long> roleRefs,
            AccountInfo accountInfo,
            long createdBy) {
        var newInstance = new Account(null, username, email, mobilePhone,
                Password.newPassword(password, passwordConfirmation), status, accountInfo, roleRefs,
                createdBy, null, 0);
        newInstance.addDomainEvent(new AccountCreated(newInstance));
        return newInstance;
    }

    public static Account register(
            String username,
            String email,
            String mobilePhone,
            Password password,
            AccountStatus status,
            Set<Long> roleRefs,
            AccountInfo accountInfo) {
        var newInstance = new Account(null, username, email, mobilePhone, password, status, accountInfo, roleRefs,
                0L, null, 0);
        newInstance.addDomainEvent(new AccountCreated(newInstance));
        return newInstance;
    }

    @Override
    public Optional<Long> getId() {
        return Optional.ofNullable(id);
    }

    public Set<Long> getRoleIds() {
        return Set.copyOf(this.roleRefs);
    }

    Password getPassword() {
        return password;
    }
}
