package xyz.zhouxy.plusone.system.domain.model.account;

import java.util.Objects;
import java.util.regex.Pattern;

import javax.annotation.Nonnull;

import org.springframework.util.Assert;

import xyz.zhouxy.plusone.constant.RegexConsts;
import xyz.zhouxy.plusone.domain.IValueObject;
import xyz.zhouxy.plusone.exception.PlusoneException;
import xyz.zhouxy.plusone.system.util.PasswordUtil;

/**
 * 值对象：加盐加密的密码
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
public class Password implements IValueObject {

    private static final Pattern PATTERN = Pattern.compile(RegexConsts.PASSWORD);
    private static final String DEFAULT_PASSWORD = "A1b2C3d4";

    @Nonnull
    private final String passwordVal;
    @Nonnull
    private final String saltVal;

    private Password(String password) {
        if (password == null) {
            throw new IllegalArgumentException("密码不能为空");
        }
        if (!PATTERN.matcher(password).matches()) {
            throw new IllegalArgumentException("密码格式不符合要求");
        }
        var salt = PasswordUtil.generateRandomSalt();
        if (salt == null) {
            throw new PlusoneException(9999999, "未知错误：生成随机盐失败");
        }
        this.saltVal = salt;
        this.passwordVal = PasswordUtil.hashPassword(password, salt);
    }

    private Password(String password, String salt) {
        if (password == null || salt == null) {
            throw new IllegalArgumentException("password 和 salt 不能为空");
        }
        this.passwordVal = password;
        this.saltVal = salt;
    }

    public static Password of(String password, String salt) {
        return new Password(password, salt);
    }

    public static Password newPassword(String newPassword, String passwordConfirmation) {
        Assert.isTrue(Objects.equals(newPassword, passwordConfirmation), "两次输入的密码不一致");
        return newPassword(newPassword);
    }

    public static Password newPassword(String newPassword) {
        return new Password(newPassword);
    }

    public boolean check(String password) {
        if (password == null) {
            throw new IllegalArgumentException("password 不能为空");
        }
        Assert.hasText(password, "密码不能为空");
        return Objects.equals(this.passwordVal, PasswordUtil.hashPassword(password, this.saltVal));
    }

    public String value() {
        return passwordVal;
    }

    public String getSalt() {
        return saltVal;
    }

    public static Password newDefaultPassword() {
        return newPassword(DEFAULT_PASSWORD);
    }

    @Override
    public String toString() {
        return "********";
    }
}
