package xyz.zhouxy.plusone.system.domain.model.dict;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

import lombok.ToString;
import xyz.zhouxy.plusone.domain.AggregateRoot;
import xyz.zhouxy.plusone.domain.IWithLabel;
import xyz.zhouxy.plusone.domain.IWithVersion;

/**
 * 聚合根：数据字典
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@ToString
public class Dict extends AggregateRoot<Long> implements IWithLabel, IWithVersion {

    private Long id;
    private String dictType;
    private String dictLabel;

    private Map<Integer, DictValue> values = new HashMap<>();

    private long version;

    // ==================== 领域逻辑 ====================

    public void addValue(int key, String label) {
        if (this.values.containsKey(key)) {
            throw new IllegalArgumentException(String.format("字典 %s 已存在值：%d", dictType, key));
        }
        this.values.put(key, DictValue.of(key, label));
    }

    public void removeValue(int key) {
        this.values.remove(key);
    }

    public void updateDict(String dictType, String dictLabel, Map<Integer, String> keyLabelMap) {
        this.dictType = dictType;
        this.dictLabel = dictLabel;
        var valueKeys = this.values.keySet();
        for (Integer key : valueKeys) {
            if (!keyLabelMap.containsKey(key)) {
                this.values.remove(key);
            }
        }
        keyLabelMap.forEach((Integer key, String label) -> {
            var temp = this.values.get(key);
            if (temp != null) {
                temp.label = label;
            } else {
                this.values.put(key, DictValue.of(key, label));
            }
        });
    }

    // ==================== 实例化 ====================

    Dict(Long id, String dictType, String dictLabel, Set<DictValue> values, long version) {
        this.id = id;
        this.dictType = dictType;
        this.dictLabel = dictLabel;
        values.forEach(dictValue -> this.values.put(dictValue.key, dictValue));
        this.version = version;
    }

    public static Dict newInstance(
            String dictType,
            String dictLabel) {
        return new Dict(null, dictType, dictLabel, Collections.emptySet(), 0);
    }

    public static Dict newInstance(
            String dictType,
            String dictLabel,
            Set<DictValue> values) {
        return new Dict(null, dictType, dictLabel, values, 0);
    }

    public static Dict newInstance(
            String dictType,
            String dictLabel,
            Map<Integer, String> keyLabelMap) {
        var values = buildDictValues(keyLabelMap);
        return new Dict(null, dictType, dictLabel, values, 0);
    }

    private static Set<DictValue> buildDictValues(Map<Integer, String> keyLabelMap) {
        Set<DictValue> dictValues = new HashSet<>(keyLabelMap.size());
        keyLabelMap.forEach((Integer key, String label) -> dictValues.add(DictValue.of(key, label)));
        return dictValues;
    }

    // ==================== getters ====================

    @Override
    public Optional<Long> getId() {
        return Optional.ofNullable(id);
    }

    public String getDictType() {
        return dictType;
    }

    @Override
    public String getLabel() {
        return this.dictLabel;
    }

    public Set<DictValue> getValues() {
        return this.values.values().stream().collect(Collectors.toSet());
    }

    @Override
    public long getVersion() {
        return this.version;
    }

    public int count() {
        return this.values.size();
    }
}
