package xyz.zhouxy.plusone.system.domain.model.account;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.Objects;

import javax.annotation.Nonnull;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.stereotype.Repository;

import cn.hutool.core.util.IdUtil;
import xyz.zhouxy.plusone.jdbc.JdbcRepositorySupport;
import xyz.zhouxy.plusone.util.AssertResult;

/**
 * AccountRepository 实现类
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Repository
public class AccountRepositoryImpl extends JdbcRepositorySupport<Account, Long> implements AccountRepository {

    private final AccountRoleRefDAO accountRoleDAO;

    public AccountRepositoryImpl(@Nonnull NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        super(namedParameterJdbcTemplate);
        this.accountRoleDAO = new AccountRoleRefDAO(namedParameterJdbcTemplate);
    }

    @Override
    protected final void doDelete(@Nonnull Account entity) {
        int i = update("""
                UPDATE sys_account SET deleted = id, "version" = "version" + 1
                WHERE id = :id AND deleted = 0 AND "version" = :version
                """,
                new MapSqlParameterSource()
                        .addValue("id", entity.getId().orElseThrow())
                        .addValue("version", entity.getVersion()));
        AssertResult.update(i, 1);
    }

    @Override
    protected final Account doFindById(@Nonnull Long id) {
        return queryForObject("""
                SELECT
                    id, email, mobile_phone, username, "password", salt, avatar, sex, nickname, status,
                    created_by, updated_by, "version"
                FROM sys_account
                WHERE id = :id AND deleted = 0
                """,
                new MapSqlParameterSource("id", id));
    }

    @Override
    public Account findByEmail(Email email) {
        return queryForObject("""
                SELECT
                    id, email, mobile_phone, username, "password", salt, avatar, sex, nickname, status,
                    created_by, updated_by, "version"
                FROM sys_account
                WHERE email = :email AND deleted = 0
                """,
                new MapSqlParameterSource("email", email.value()));
    }

    @Override
    public Account findByMobilePhone(MobilePhone mobilePhone) {
        return queryForObject("""
                SELECT
                    id, email, mobile_phone, username, "password", salt, avatar, sex, nickname, status,
                    created_by, updated_by, "version"
                FROM sys_account
                WHERE mobile_phone = :mobilePhone AND deleted = 0
                """,
                new MapSqlParameterSource("mobilePhone", mobilePhone.value()));
    }

    @Override
    public Account findByUsername(Username username) {
        return queryForObject("""
                SELECT
                    id, email, mobile_phone, username, "password", salt, avatar, sex, nickname, status,
                    created_by, updated_by, "version"
                FROM sys_account
                WHERE username = :username AND deleted = 0
                """,
                new MapSqlParameterSource("username", username.value()));
    }

    @Override
    public boolean exists(Long id) {
        return queryExists("SELECT 1 FROM sys_account WHERE id = :id AND deleted = 0 LIMIT 1",
                new MapSqlParameterSource("id", id));
    }

    @Override
    public boolean existsUsername(Username username) {
        return queryExists("SELECT 1 FROM sys_account WHERE username = :username AND deleted = 0 LIMIT 1",
                new MapSqlParameterSource("username", username.value()));
    }

    @Override
    public boolean existsEmail(Email email) {
        return queryExists("SELECT 1 FROM sys_account WHERE email = :email AND deleted = 0 LIMIT 1",
                new MapSqlParameterSource("email", email.value()));
    }

    @Override
    public boolean existsMobilePhone(MobilePhone mobilePhone) {
        return queryExists("SELECT 1 FROM sys_account WHERE mobile_phone = :mobile_phone AND deleted = 0 LIMIT 1",
                new MapSqlParameterSource("mobile_phone", mobilePhone.value()));
    }

    @Override
    public Collection<Account> findByRoleId(Long roleId) {
        return queryForList("""
                SELECT
                    a.id, a.email, a.mobile_phone, a.username, a."password", a.salt,
                    a.avatar, a.sex, a.nickname, a.status,
                    a.created_by, a.updated_by, a."version"
                FROM sys_account a
                LEFT JOIN sys_account_role ar ON a.id = ar.account_id
                WHERE ar.role_id = :roleId AND a.deleted = 0
                """,
                new MapSqlParameterSource("roleId", roleId));
    }

    @Override
    protected final Account doInsert(@Nonnull Account entity) {
        long id = IdUtil.getSnowflakeNextId();
        int i = update("""
                INSERT INTO sys_account
                (id, email, mobile_phone, username, "password", salt, avatar, sex, nickname, status,
                 created_by, create_time)
                VALUES
                (:id, :email, :mobilePhone, :username, :password, :salt, :avatar, :sex, :nickname, :status,
                 :createdBy, :createTime)
                """,
                generateParamSource(id, entity));
        AssertResult.update(i, 1);
        this.accountRoleDAO.insertAccountRoleRefs(id, entity.getRoleIds());
        return entity;
    }

    @Override
    protected final Account doUpdate(@Nonnull Account entity) {
        int i = update("""
                UPDATE sys_account
                SET "email"        = :email,
                    "mobile_phone" = :mobilePhone,
                    "username"     = :username,
                    "password"     = :password,
                    "salt"         = :salt,
                    "avatar"       = :avatar,
                    "sex"          = :sex,
                    "nickname"     = :nickname,
                    "status"       = :status,
                    "updated_by"   = :updatedBy,
                    "update_time"  = :updateTime,
                    "version"      = "version" + 1
                WHERE id = :id AND deleted = 0 AND "version" = :version
                """,
                generateParamSource(entity));
        AssertResult.update(i, 1);
        this.accountRoleDAO.saveAccountRoleRefs(entity);
        return entity;
    }

    @Override
    protected final Account mapRow(ResultSet rs) throws SQLException {
        long accountId = rs.getLong("id");
        AccountInfo accountInfo = AccountInfo.of(
                rs.getString("nickname"),
                rs.getString("avatar"),
                Sex.of(rs.getInt("sex")));
        return new Account(
                accountId,
                rs.getString("username"),
                rs.getString("email"),
                rs.getString("mobile_phone"),
                Password.of(rs.getString("password"), rs.getString("salt")),
                AccountStatus.of(rs.getInt("status")),
                accountInfo,
                this.accountRoleDAO.selectRoleIdsByAccountId(accountId),
                rs.getLong("created_by"),
                rs.getLong("updated_by"),
                rs.getLong("version"));
    }

    @Override
    protected final SqlParameterSource generateParamSource(Long id, @Nonnull Account entity) {
        LocalDateTime now = LocalDateTime.now();
        AccountInfo accountInfo = entity.getAccountInfo();
        return new MapSqlParameterSource()
                .addValue("id", id)
                .addValue("email", Objects.nonNull(entity.getEmail()) ? entity.getEmail().value() : null)
                .addValue("mobilePhone",
                        Objects.nonNull(entity.getMobilePhone()) ? entity.getMobilePhone().value() : null)
                .addValue("username", entity.getUsername().value())
                .addValue("password", entity.getPassword().value())
                .addValue("salt", entity.getPassword().getSalt())
                .addValue("avatar", accountInfo.getAvatar().toString())
                .addValue("sex", accountInfo.getSex().getValue())
                .addValue("nickname",
                        Objects.nonNull(accountInfo.getNickname()) ? accountInfo.getNickname().value() : null)
                .addValue("status", entity.getStatus().getValue())
                .addValue("createdBy", entity.getCreatedBy())
                .addValue("createTime", now)
                .addValue("updatedBy", entity.getUpdatedBy())
                .addValue("updateTime", now)
                .addValue("version", entity.getVersion());
    }
}
