package xyz.zhouxy.plusone.system.domain.model.dict;

import static xyz.zhouxy.plusone.system.constant.AuthLogic.adminAuthLogic;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDateTime;
import java.util.List;

import javax.annotation.Nonnull;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.stereotype.Repository;

import cn.hutool.core.util.IdUtil;
import xyz.zhouxy.plusone.jdbc.JdbcRepositorySupport;
import xyz.zhouxy.plusone.util.AssertResult;

/**
 * DictRepository 实现类
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Repository
public class DictRepositoryImpl extends JdbcRepositorySupport<Dict, Long> implements DictRepository {

    private final DictValueDAO dictValueDAO;

    public DictRepositoryImpl(@Nonnull NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        super(namedParameterJdbcTemplate);
        this.dictValueDAO = new DictValueDAO(namedParameterJdbcTemplate);
    }

    @Override
    public Dict doFindById(@Nonnull Long id) {
        return queryForObject("SELECT id, dict_type, dict_label, \"version\" WHERE id = :id AND deleted = 0",
                new MapSqlParameterSource("id", id));
    }

    @Override
    protected final Dict doInsert(@Nonnull Dict entity) {
        long id = IdUtil.getSnowflakeNextId();
        int i = update("""
                INSERT INTO sys_dict_type (dict_type, dict_label, create_time, created_by)
                VALUES (:dictType, :dictLabel, :createTime, :createdBy)
                """,
                generateParamSource(id, entity));
        AssertResult.update(i, 1);
        this.dictValueDAO.insertDictValues(id, entity);
        return find(id);
    }

    @Override
    protected final Dict doUpdate(@Nonnull Dict entity) {
        int i = update("""
                UPDATE sys_dict_type
                SET dict_type   = :dictType,
                    dict_label  = :dictLabel,
                    update_time = :updateTime,
                    updated_by  = :updatedBy,
                    "version"   = "version" + 1
                WHERE id = :id AND deleted = 0 AND "version" = :version
                """,
                generateParamSource(entity));
        AssertResult.update(i, 1);
        this.dictValueDAO.updateDictValues(entity);
        return find(entity.getId().orElseThrow());
    }

    @Override
    protected final void doDelete(@Nonnull Dict entity) {
        int i = update("""
                UPDATE sys_dict_type SET deleted = id, "version" = "version" + 1
                WHERE id = :id AND deleted = 0 AND "version" = :version
                """,
                generateParamSource(entity));
        AssertResult.update(i, 1);
    }

    @Override
    public boolean exists(Long id) {
        return queryExists("SELECT 1 FROM sys_dict_type WHERE id = :id AND deleted = 0 LIMIT 1",
                new MapSqlParameterSource("id", id));
    }

    @Override
    public List<Dict> findAll() {
        return queryForList("SELECT id, dict_type, dict_label, \"version\" WHERE deleted = 0");
    }

    @Override
    protected final Dict mapRow(ResultSet rs) throws SQLException {
        long id = rs.getLong("id");
        return new Dict(
                id,
                rs.getString("dict_type"),
                rs.getString("dict_label"),
                this.dictValueDAO.selectDictValuesByDictId(id),
                rs.getLong("version"));
    }

    @Override
    protected final SqlParameterSource generateParamSource(Long id, @Nonnull Dict entity) {
        LocalDateTime now = LocalDateTime.now();
        long loginId = adminAuthLogic.getLoginIdAsLong();
        return new MapSqlParameterSource()
                .addValue("dictType", entity.getDictType())
                .addValue("dictLabel", entity.getLabel())
                .addValue("createTime", now)
                .addValue("createdBy", loginId)
                .addValue("updateTime", now)
                .addValue("updatedBy", loginId)
                .addValue("id", id)
                .addValue("version", entity.getVersion());
    }
}
