package xyz.zhouxy.plusone.system.domain.model.menu;

import static xyz.zhouxy.plusone.system.constant.AuthLogic.adminAuthLogic;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import javax.annotation.Nonnull;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import cn.hutool.core.util.IdUtil;
import xyz.zhouxy.plusone.jdbc.JdbcEntityDaoSupport;

/**
 * {@link Action} 的数据访问对象
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 * @date 2022-10-31 12:31:49
 */
class ActionDAO extends JdbcEntityDaoSupport<Action, Long> {

    ActionDAO(@Nonnull NamedParameterJdbcTemplate jdbc) {
        super(jdbc);
    }

    void saveActions(Long menuId, List<Action> actions) {
        // 删除要删除的权限
        Collection<Long> ids = actions.stream()
                .filter(action -> action.getId().isPresent())
                .map(action -> action.getId().orElseThrow())
                .collect(Collectors.toSet());
        if (!ids.isEmpty()) {
            this.jdbc.update(
                    "UPDATE sys_action SET deleted = id WHERE resource = :resource AND id NOT IN (:ids) AND deleted = 0",
                    new MapSqlParameterSource()
                            .addValue("resource", menuId)
                            .addValue("ids", ids));
        }

        // 更新存在的数据
        this.jdbc.batchUpdate("""
                UPDATE sys_action
                SET resource = :resource,
                    identifier = :identifier,
                    label = :label,
                    update_time = :updateTime,
                    updated_by = :updatedBy
                WHERE id = :id AND deleted = 0
                """,
                actions.stream()
                        .filter(action -> action.getId().isPresent())
                        .map(action -> generateParamSource(menuId, action))
                        .toArray(MapSqlParameterSource[]::new));

        // 插入新添加的数据
        this.jdbc.batchUpdate("""
                INSERT INTO sys_action
                (id, resource, identifier, "label", create_time, created_by)
                VALUES
                (:id, :resource, :identifier, :label, :createTime, :createdBy)
                """,
                actions.stream()
                        .filter(action -> action.getId().isEmpty())
                        .map(action -> generateParamSource(menuId, IdUtil.getSnowflakeNextId(), action))
                        .toArray(MapSqlParameterSource[]::new));
    }

    List<Action> selectActionsByMenuId(long menuId) {
        return queryForList("""
                SELECT a.id, m.resource, a.identifier, a.label
                FROM sys_action a
                JOIN (SELECT id, resource FROM sys_menu WHERE id = :menuId AND deleted = 0) m ON a.resource = m.id
                WHERE a.deleted = 0
                """, new MapSqlParameterSource("menuId", menuId));
    }

    Collection<Action> selectActionsByIdIn(Collection<Long> actionIds) {
        if (Objects.isNull(actionIds) || actionIds.isEmpty()) {
            return Collections.emptyList();
        }
        return queryForList("""
                SELECT a.id, m.resource, a.identifier, a.label
                FROM sys_action a
                LEFT JOIN sys_menu m ON a.resource = m.id
                WHERE a.id IN (:actionIds) AND a.deleted = 0
                """, new MapSqlParameterSource("actionIds", actionIds));
    }

    private SqlParameterSource generateParamSource(Long menuId, Action action) {
        return generateParamSource(menuId, action.getId().orElseThrow(), action);
    }

    private SqlParameterSource generateParamSource(Long menuId, Long actionId, Action action) {
        long loginId = adminAuthLogic.getLoginIdAsLong();
        LocalDateTime now = LocalDateTime.now();
        return new MapSqlParameterSource("id", actionId)
                .addValue("resource", menuId)
                .addValue("identifier", action.getIdentifier())
                .addValue("label", action.getLabel())
                .addValue("createTime", now)
                .addValue("createdBy", loginId)
                .addValue("updateTime", now)
                .addValue("updatedBy", loginId);
    }

    @Override
    protected Action mapRow(ResultSet rs) throws SQLException {
        return Action.of(
                rs.getLong("id"),
                rs.getString("resource"),
                rs.getString("identifier"),
                rs.getString("label"));
    }
}
