package xyz.zhouxy.plusone.system.domain.model.menu;

import static xyz.zhouxy.plusone.system.constant.AuthLogic.adminAuthLogic;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.Collections;
import java.util.Objects;

import javax.annotation.Nonnull;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import cn.hutool.core.util.IdUtil;
import xyz.zhouxy.plusone.constant.EntityStatus;
import xyz.zhouxy.plusone.jdbc.JdbcRepositorySupport;
import xyz.zhouxy.plusone.system.domain.model.menu.Menu.MenuType;
import xyz.zhouxy.plusone.util.AssertResult;
import xyz.zhouxy.plusone.util.EnumUtil;

/**
 * MenuRepository 实现类
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Repository
public class MenuRepositoryImpl extends JdbcRepositorySupport<Menu, Long> implements MenuRepository {

    private final ActionDAO actionDAO;

    MenuRepositoryImpl(@Nonnull NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        super(namedParameterJdbcTemplate);
        this.actionDAO = new ActionDAO(namedParameterJdbcTemplate);
    }

    @Override
    protected final Menu doFindById(@Nonnull Long id) {
        return queryForObject("""
                SELECT
                    id, parent_id, "type", "name", "path", title, icon, hidden, order_number, status, remarks,
                    component, "cache", resource, "version"
                FROM sys_menu
                WHERE id = :id AND deleted = 0
                """,
                new MapSqlParameterSource("id", id));
    }

    @Override
    protected final Menu doInsert(@Nonnull Menu entity) {
        long id = IdUtil.getSnowflakeNextId();
        String sql = """
                INSERT INTO sys_menu (
                    id, parent_id, "type", name, "path", title, icon, hidden, order_number, status, remarks,
                    component, "cache", resource, create_time, created_by)
                VALUES
                    (:id, :parentId, :type, :name, :path, :title, :icon, :hidden, :orderNumber, :status, :remarks,
                     :component, :cache, :resource, :createTime, :createdBy)
                """;
        MapSqlParameterSource paramSource = generateParamSource(id, entity);
        int i = update(sql, paramSource);
        AssertResult.update(i, 1);
        this.actionDAO.saveActions(id, entity.getActions());
        return entity;
    }

    @Override
    protected final Menu doUpdate(@Nonnull Menu entity) {
        String sql = """
                UPDATE sys_menu
                SET "parent_id"    = :parentId,
                    "type"         = :type,
                    "name"         = :name,
                    "path"         = :path,
                    "title"        = :title,
                    "icon"         = :icon,
                    "hidden"       = :hidden,
                    "order_number" = :orderNumber,
                    "status"       = :status,
                    "remarks"      = :remarks,
                    "component"    = :component,
                    "cache"        = :cache,
                    "resource"     = :resource,
                    "update_time"  = :updateTime,
                    "updated_by"   = :updatedBy,
                    "version"      = "version" + 1
                WHERE id = :id AND deleted = 0 AND "version" = :version
                """;

        // 更新菜单
        int i = update(sql, generateParamSource(entity));
        AssertResult.update(i, 1);

        // 保存权限
        Long id = entity.getId().orElseThrow();
        this.actionDAO.saveActions(id, entity.getActions());
        return entity;
    }

    @Override
    protected final void doDelete(@Nonnull Menu entity) {
        int i = update("""
                UPDATE sys_menu SET deleted = id, "version" = "version" + 1
                WHERE id = :id AND deleted = 0 AND "version" = :version
                """,
                new MapSqlParameterSource("id", entity.getId().orElseThrow())
                        .addValue("version", entity.getVersion()));
        AssertResult.update(i, 1);
    }

    @Override
    public boolean exists(Long id) {
        return queryExists("SELECT 1 FROM sys_menu WHERE id = :id AND deleted = 0 LIMIT 1",
                new MapSqlParameterSource("id", id));
    }

    @Override
    public Collection<Menu> findByIdIn(Collection<Long> ids) {
        if (Objects.isNull(ids) || ids.isEmpty()) {
            return Collections.emptyList();
        }
        return queryForList("""
                SELECT
                    id, parent_id, "type", "name", "path", title, icon, hidden, order_number, status, remarks,
                    component, "cache", resource, "version"
                FROM sys_menu
                WHERE id IN (:ids) AND deleted = 0
                """,
                new MapSqlParameterSource("ids", ids));
    }

    @Override
    public Collection<Action> findPermissionsByIdIn(Collection<Long> permissionIds) {
        return this.actionDAO.selectActionsByIdIn(permissionIds);
    }

    @Override
    public Collection<Menu> queryByRoleId(Long roleId) {
        return queryForList("""
                SELECT
                    m.id, m.parent_id, m."type", m."name", m."path", m.title, m.icon, m.hidden, m.order_number,
                    m.status, m.remarks, m.component, m."cache", m.resource, m."version"
                FROM sys_menu AS m
                LEFT JOIN sys_role_menu AS rm ON m.id = rm.menu_id
                WHERE rm.role_id = :roleId AND r.deleted = 0
                """,
                new MapSqlParameterSource("roleId", roleId));
    }

    @Override
    protected final Menu mapRow(ResultSet rs) throws SQLException {
        long menuId = rs.getLong("id");
        return new Menu(
                EnumUtil.valueOf(MenuType.class, rs.getInt("type")),
                menuId,
                rs.getLong("parent_id"),
                rs.getString("name"),
                rs.getString("path"),
                rs.getString("title"),
                rs.getString("icon"),
                rs.getBoolean("hidden"),
                rs.getInt("order_number"),
                EntityStatus.of(rs.getInt("status")),
                rs.getString("remarks"),
                rs.getString("component"),
                rs.getBoolean("cache"),
                rs.getString("resource"),
                this.actionDAO.selectActionsByMenuId(menuId),
                rs.getLong("version"));
    }

    @Override
    protected final MapSqlParameterSource generateParamSource(Long id, @Nonnull Menu entity) {
        LocalDateTime now = LocalDateTime.now();
        long loginId = adminAuthLogic.getLoginIdAsLong();
        return new MapSqlParameterSource()
                .addValue("id", id)
                .addValue("parentId", entity.getParentId())
                .addValue("type", entity.getType().value())
                .addValue("name", entity.getName())
                .addValue("path", entity.getPath())
                .addValue("title", entity.getTitle())
                .addValue("icon", entity.getIcon())
                .addValue("hidden", entity.isHidden())
                .addValue("orderNumber", entity.getOrderNumber())
                .addValue("status", entity.getStatus().getValue())
                .addValue("remarks", entity.getRemarks())
                .addValue("component", entity.getComponent())
                .addValue("cache", entity.getCache())
                .addValue("resource", entity.getResource())
                .addValue("createTime", now)
                .addValue("createdBy", loginId)
                .addValue("updateTime", now)
                .addValue("updatedBy", loginId)
                .addValue("version", entity.getVersion());
    }
}
