package xyz.zhouxy.plusone.system.domain.model.role;

import static xyz.zhouxy.plusone.system.constant.AuthLogic.adminAuthLogic;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDateTime;
import java.util.Collection;

import javax.annotation.Nonnull;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import cn.hutool.core.util.IdUtil;
import xyz.zhouxy.plusone.constant.EntityStatus;
import xyz.zhouxy.plusone.jdbc.JdbcRepositorySupport;
import xyz.zhouxy.plusone.util.AssertResult;

/**
 * RoleRepository 实现类
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Repository
public class RoleRepositoryImpl extends JdbcRepositorySupport<Role, Long> implements RoleRepository {

    private final RoleMenuRefDAO roleMenuRefDAO;
    private final RolePermissionRefDAO rolePermissionRefDAO;

    RoleRepositoryImpl(@Nonnull NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        super(namedParameterJdbcTemplate);
        this.roleMenuRefDAO = new RoleMenuRefDAO(namedParameterJdbcTemplate);
        this.rolePermissionRefDAO = new RolePermissionRefDAO(namedParameterJdbcTemplate);
    }

    @Override
    protected final Role doFindById(@Nonnull Long id) {
        return queryForObject("""
                SELECT "id","name","identifier","status","remarks","version"
                FROM "sys_role"
                WHERE id = :id AND deleted = 0
                """, new MapSqlParameterSource("id", id));
    }

    @Override
    protected final void doDelete(@Nonnull Role entity) {
        int i = update("""
                UPDATE sys_account SET deleted = id, "version" = "version" + 1
                WHERE id = :id AND deleted = 0 AND "version" = :version
                """,
                new MapSqlParameterSource("id", entity.getId().orElseThrow())
                        .addValue("version", entity.getVersion()));
        AssertResult.update(i, 1);
    }

    @Override
    public boolean exists(Long id) {
        return queryExists("SELECT 1 FROM sys_role WHERE id = :id AND deleted = 0 LIMIT 1",
                new MapSqlParameterSource("id", id));
    }

    @Override
    protected final Role doInsert(@Nonnull Role entity) {
        long id = IdUtil.getSnowflakeNextId();
        int i = update("""
                INSERT INTO sys_role
                (id, "name", identifier, status, remarks, create_time, created_by)
                VALUES
                (:id, :name, :identifier, :status, :remarks, :createTime, :createdBy)
                """, generateParamSource(id, entity));
        AssertResult.update(i, 1);
        this.roleMenuRefDAO.saveRoleMenuRefs(id, entity);
        this.rolePermissionRefDAO.saveRolePermissionRefs(id, entity);
        return entity;
    }

    @Override
    protected final Role doUpdate(@Nonnull Role entity) {
        int i = update("""
                UPDATE sys_role
                SET "name"        = :name,
                    "identifier"  = :identifier,
                    "status"      = :status,
                    "remarks"     = :remarks,
                    "update_time" = :updateTime,
                    "updated_by"  = :updatedBy,
                    "version"     = "version" + 1
                WHERE id = :id AND deleted = 0 AND "version" = :version
                """, generateParamSource(entity));
        AssertResult.update(i, 1);

        Long id = entity.getId().orElseThrow();
        this.roleMenuRefDAO.clearRoleMenuRefs(entity);
        this.roleMenuRefDAO.saveRoleMenuRefs(id, entity);
        this.rolePermissionRefDAO.clearRolePermissionRefs(entity);
        this.rolePermissionRefDAO.saveRolePermissionRefs(id, entity);
        return entity;
    }

    @Override
    public Collection<Role> findByAccountId(Long accountId) {
        return queryForList("""
                SELECT r."id", r."name", r."identifier", r."status", r."remarks", r."version"
                FROM sys_role AS r
                LEFT JOIN sys_account_role AS ar ON r.id = ar.role_id
                WHERE ar.account_id = :accountId AND r.deleted = 0
                """, new MapSqlParameterSource("accountId", accountId));
    }

    @Override
    protected final Role mapRow(ResultSet rs) throws SQLException {
        long roleId = rs.getLong("id");
        return new Role(
                roleId,
                rs.getString("name"),
                rs.getString("identifier"),
                EntityStatus.of(rs.getInt("status")),
                rs.getString("remarks"),
                this.roleMenuRefDAO.selectMenuRefsByRoleId(roleId),
                this.rolePermissionRefDAO.selectPermissionRefsByRoleId(roleId),
                rs.getLong("version"));
    }

    @Override
    protected final MapSqlParameterSource generateParamSource(Long id, @Nonnull Role entity) {
        LocalDateTime now = LocalDateTime.now();
        long loginId = adminAuthLogic.getLoginIdAsLong();
        return new MapSqlParameterSource()
                .addValue("id", id)
                .addValue("name", entity.getName())
                .addValue("identifier", entity.getIdentifier())
                .addValue("status", entity.getStatus().getValue())
                .addValue("remarks", entity.getRemarks())
                .addValue("createTime", now)
                .addValue("createdBy", loginId)
                .addValue("updateTime", now)
                .addValue("updatedBy", loginId)
                .addValue("version", entity.getVersion());
    }
}
